package io.tswf.minecraft.blockventure.core.world.chunk.persist

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.core.transmutation.expansion.IServerWorldPersistedChunksAccess
import it.unimi.dsi.fastutil.longs.LongOpenHashSet
import net.minecraft.server.world.ServerWorld
import net.minecraft.util.math.ChunkPos

@Slf4j
@CompileStatic
class ChunkPersistService {

    static boolean shouldLogPersistOperations = System.findGlobalProperty("blockventure.world.chunk.persist.logs.enabled")
        .map { it.toBoolean() }
        .orElse(true)


    static void persistChunk(ServerWorld world, int chunkX, int chunkZ) {
        def positions = getSerializedChunkPositions(world)
        def chunkPos = ChunkPos.toLong(chunkX, chunkZ)

        if (positions.add(chunkPos)) {
            if (shouldLogPersistOperations) {
                log.info("Persisting chunk ${chunkX} ${chunkZ} in world ${world.dimensionKey.value}...")
            }
        }
    }

    static void freeChunk(ServerWorld world, int chunkX, int chunkZ) {
        def positions = getSerializedChunkPositions(world)
        def chunkPos = ChunkPos.toLong(chunkX, chunkZ)

        if (positions.remove(chunkPos)) {
            if (shouldLogPersistOperations) {
                log.info("Free chunk ${chunkX} ${chunkZ} in world ${world.dimensionKey.value}...")
            }
        }
    }

    static boolean isChunkPersisted(ServerWorld world, int chunkX, int chunkZ) {
        return isChunkPersisted(world, ChunkPos.toLong(chunkX, chunkZ))
    }

    static boolean isChunkPersisted(ServerWorld world, long serializedChunkPos) {
        def positions = getSerializedChunkPositions(world)
        return positions.contains(serializedChunkPos)
    }

    static LongOpenHashSet getSerializedChunkPositions(ServerWorld serverWorld) {
        return ((IServerWorldPersistedChunksAccess) serverWorld).serializedPersistedChunkPositions
    }
}
